@@ -2,14 +2,15 @@
 /**
  * This file contains basic classes for representing (query) descriptions in
  * the SMW API.
- *
+ * @file
+ * @ingroup SMWQuery
  * @author Markus Krötzsch
  */
 
 /**
  * Container class for request for printout, as used in queries to
  * obtain additional information for the retrieved results.
- * @note: AUTOLOADED
+ * @ingroup SMWQuery
  */
 class SMWPrintRequest {
 	const PRINT_CATS = 0; // print all direct cateories of the current element
@@ -19,7 +20,7 @@ class SMWPrintRequest {
 
 	protected $m_mode; // type of print request
 	protected $m_label; // string for labelling results, contains no markup
-	protected $m_title; // title object to which print request refers (if any)
+	protected $m_data; // data entries specifyin gwhat was requested (mixed type)
 	protected $m_typeid = false; // id of the datatype of the printed objects, if applicable
 	protected $m_outputformat; // output format string for formatting results, if applicable
 
@@ -27,19 +28,22 @@ class SMWPrintRequest {
 	 * Create a print request.
 	 * @param $mode a constant defining what to printout
 	 * @param $label the string label to describe this printout
-	 * @param $title optional Title object that specifies the request (usually some property)
-	 * @param $datavalue optional SMWDataValue container that sets parameters for printing data values (e.g. the unit)
+	 * @param $data optional data for specifying some request, might be a property object, title, or something else; interpretation depends on $mode
+	 * @param $outputformat optional string for specifying an output format, e.g. an output unit
 	 */
-	public function SMWPrintRequest($mode, $label, $title = NULL, $outputformat = '') {
+	public function __construct($mode, $label, $data = NULL, $outputformat = '') {
 		$this->m_mode = $mode;
 		$this->m_label = $label;
-		$this->m_title = $title;
+		$this->m_data = $data;
 		$this->m_outputformat = $outputformat;
 		if ( ($mode == SMWPrintRequest::PRINT_CCAT) && ($outputformat === '') ) {
 			$this->m_outputformat = 'x'; // changed default for Boolean case
 		}
+		if ($this->m_data instanceof SMWDataValue) {
+			$this->m_data->setCaption($label);
+		}
 	}
-	
+
 	public function getMode() {
 		return $this->m_mode;
 	}
@@ -58,9 +62,12 @@ class SMWPrintRequest {
 			return htmlspecialchars($this->m_label);
 		}
 		switch ($this->m_mode) {
-			case SMWPrintRequest::PRINT_CATS: return htmlspecialchars($this->m_label); // TODO: link to Special:Categories
-			case SMWPrintRequest::PRINT_PROP: case SMWPrintRequest::PRINT_CCAT:
-				return $linker->makeLinkObj($this->m_title, htmlspecialchars($this->m_label));
+			case SMWPrintRequest::PRINT_CATS: 
+				return htmlspecialchars($this->m_label); // TODO: link to Special:Categories
+			case SMWPrintRequest::PRINT_CCAT:
+				return $linker->makeLinkObj($this->m_data->getTitle(), htmlspecialchars($this->m_label));
+			case SMWPrintRequest::PRINT_PROP:
+				return $this->m_data->getShortHTMLText($linker);
 			case SMWPrintRequest::PRINT_THIS: default: return htmlspecialchars($this->m_label);
 		}
 		
@@ -75,8 +82,10 @@ class SMWPrintRequest {
 		} else {
 			switch ($this->m_mode) {
 				case SMWPrintRequest::PRINT_CATS: return $this->m_label; // TODO: link to Special:Categories
-				case SMWPrintRequest::PRINT_PROP: case SMWPrintRequest::PRINT_CCAT:
-					return '[[:' . $this->m_title->getPrefixedText() . '|' . $this->m_label . ']]';
+				case SMWPrintRequest::PRINT_PROP:
+					return $this->m_data->getShortWikiText($linked);
+				case SMWPrintRequest::PRINT_CCAT:
+				return '[[:' . $this->m_data->getPrefixedText() . '|' . $this->m_label . ']]';
 				case SMWPrintRequest::PRINT_THIS: default: return $this->m_label;
 			}
 		}
@@ -89,18 +98,36 @@ class SMWPrintRequest {
 		}
 	}
 
+	/**
+	 * @deprecated Use SMWPrintRequest::getData().
+	 */
 	public function getTitle() {
-		return $this->m_title;
+		if ($this->m_data instanceof Title) {
+			return $this->m_data;
+		} else {
+			return NULL;
+		}
+	}
+
+	/**
+	 * Return additional data related to the print request. Might be 
+	 */
+	public function getData() {
+		return $this->m_data;
 	}
 
 	public function getOutputFormat() {
 		return $this->m_outputformat;
 	}
 
+	/**
+	 * If this print request refers to some property, return the type id of this property.
+	 * Otherwise return FALSE.
+	 */
 	public function getTypeID() {
 		if ($this->m_typeid === false) {
 			if ($this->m_mode == SMWPrintRequest::PRINT_PROP) {
-				$this->m_typeid = SMWDataValueFactory::getPropertyObjectTypeID($this->m_title);
+				$this->m_typeid = $this->m_data->getTypeID();
 			} else {
 				$this->m_typeid = '_wpg'; // return objects might be titles, but anyway
 			}
@@ -114,18 +141,19 @@ class SMWPrintRequest {
 	 */
 	public function getHash() {
 		$hash = $this->m_mode . ':';
-		if ($this->m_title !== NULL) {
-			$hash .= $this->m_title->getPrefixedText() . ':';
+		if ($this->m_data instanceof Title) {
+			$hash .= $this->m_data->getPrefixedText() . ':';
+		} elseif ($this->m_data instanceof SMWDataValue) {
+			$hash .= $this->m_data->getHash() . ':';
 		}
 		$hash .= $this->m_outputformat . ':';
 		return $hash;
 	}
 
 	/**
-	 * Serialise this object like print requests given in #ask.
+	 * Serialise this object like print requests given in \#ask.
 	 */
 	public function getSerialisation() {
-		/// TODO: do not use "= label" if label is the default anyway
 		switch ($this->m_mode) {
 			case SMWPrintRequest::PRINT_CATS:
 				global $wgContLang;
@@ -137,17 +165,19 @@ class SMWPrintRequest {
 				return $result;
 			case SMWPrintRequest::PRINT_PROP: case SMWPrintRequest::PRINT_CCAT:
 				if ($this->m_mode == SMWPrintRequest::PRINT_CCAT) {
-					$result = '?' . $this->m_title->getPrefixedText();
+					$printname = $this->m_data->getPrefixedText();
+					$result = '?' . $printname;
 					if ( $this->m_outputformat != 'x' ) {
 						$result .= '#' . $this->m_outputformat;
 					}
 				} else {
-					$result = '?' . $this->m_title->getText();
+					$printname = $this->m_data->getWikiValue();
+					$result = '?' . $printname;
 					if ( $this->m_outputformat != '' ) {
 						$result .= '#' . $this->m_outputformat;
 					}
 				}
-				if ( $this->m_title->getText() != $this->m_label ) {
+				if ( $printname != $this->m_label ) {
 					$result .= '=' . $this->m_label;
 				}
 				return $result;
@@ -158,6 +188,7 @@ class SMWPrintRequest {
 
 /**
  * Abstract base class for all descriptions.
+ * @ingroup SMWQuery
  */
 abstract class SMWDescription {
 
@@ -198,7 +229,7 @@ abstract class SMWDescription {
 	 * account.
 	 * Example: The SMWValueDescription [[Paris]] returns the single result "Paris"
 	 * but can also be used as value in [[has location::Paris]] which is preferred
-	 * over the canonical [[has location::<q>[[Paris]]</q>]].
+	 * over the canonical [[has location::\<q\>[[Paris]]\</q\>]].
 	 */
 	abstract public function getQueryString($asvalue = false);
 
@@ -222,6 +253,14 @@ abstract class SMWDescription {
 		return 0;
 	}
 
+	/**
+	 * Report on query features used in description. Return values are (sums of)
+	 * query feature constants such as SMW_PROPERTY_QUERY.
+	 */
+	public function getQueryFeatures() {
+		return 0;
+	}
+
 	/**
 	 * Recursively restrict query to a maximal size and depth as given.
 	 * Returns a possibly changed description that should be used as a replacement.
@@ -229,9 +268,9 @@ abstract class SMWDescription {
 	 * Default implementation for non-nested descriptions of size 1.
 	 * The parameter $log contains a list of all pruned conditions, updated when some
 	 * description was reduced.
-	 * NOTE: objects must not do changes on $this during pruning, since $this can be
+	 * @note Objects must not do changes on $this during pruning, since $this can be
 	 * reused in multiple places of one or many queries. Make new objects to reflect
-	 * changes.
+	 * changes!
 	 */
 	public function prune(&$maxsize, &$maxdepth, &$log) {
 		if ( ($maxsize < $this->getSize()) || ($maxdepth < $this->getDepth()) ) {
@@ -254,6 +293,7 @@ abstract class SMWDescription {
  * not used for datavalues of attributes in order to support type 
  * hinting in the API: descriptions of data are always 
  * SMWValueDescription objects.
+ * @ingroup SMWQuery
  */
 class SMWThingDescription extends SMWDescription {
 	public function getQueryString($asvalue = false) {
@@ -277,6 +317,7 @@ class SMWThingDescription extends SMWDescription {
  * Description of a single class as given by a wiki category, or of a disjunction
  * of such classes. Corresponds to (disjunctions of) atomic classes in OWL and 
  * to (unions of) classes in RDF.
+ * @ingroup SMWQuery
  */
 class SMWClassDescription extends SMWDescription {
 	protected $m_titles;
@@ -328,6 +369,14 @@ class SMWClassDescription extends SMWDescription {
 		}
 	}
 
+	public function getQueryFeatures() {
+		if (count($this->m_titles) > 1) {
+			return SMW_CATEGORY_QUERY | SMW_DISJUNCTION_QUERY;
+		} else {
+			return SMW_CATEGORY_QUERY;
+		}
+	}
+
 	public function prune(&$maxsize, &$maxdepth, &$log) {
 		if ($maxsize >= $this->getSize()) {
 			$maxsize = $maxsize - $this->getSize();
@@ -351,20 +400,21 @@ class SMWClassDescription extends SMWDescription {
 /**
  * Description of a single class as described by a concept page in the wiki. Corresponds to 
  * classes in (the EL fragment of) OWL DL, and to some extent to tree-shaped queries in SPARQL.
+ * @ingroup SMWQuery
  */
 class SMWConceptDescription extends SMWDescription {
-	protected $m_title;
+	protected $m_concept;
 
-	public function __construct($concept) {
-		$this->m_title = $concept;
+	public function __construct(Title $concept) {
+		$this->m_concept = $concept;
 	}
 
 	public function getConcept() {
-		return $this->m_title;
+		return $this->m_concept;
 	}
 
 	public function getQueryString($asvalue = false) {
-		$result = '[[' . $this->m_title->getPrefixedText() . ']]';
+		$result = '[[' . $this->m_concept->getPrefixedText() . ']]';
 		if ($asvalue) {
 			return ' &lt;q&gt;' . $result . '&lt;/q&gt; ';
 		} else {
@@ -376,6 +426,10 @@ class SMWConceptDescription extends SMWDescription {
 		return false;
 	}
 
+	public function getQueryFeatures() {
+		return SMW_CONCEPT_QUERY;
+	}
+
 	///NOTE: getSize and getDepth /could/ query the store to find the real size
 	/// of the concept. But it is not clear if this is desirable anyway, given that
 	/// caching structures may be established for retrieving concepts more quickly.
@@ -390,6 +444,7 @@ class SMWConceptDescription extends SMWDescription {
  * Corresponds to a class restriction with a special class
  * that characterises the given namespace (or at least that
  * is how one could map this to OWL etc.).
+ * @ingroup SMWQuery
  */
 class SMWNamespaceDescription extends SMWDescription {
 	protected $m_namespace;
@@ -415,6 +470,10 @@ class SMWNamespaceDescription extends SMWDescription {
 		return false;
 	}
 
+	public function getQueryFeatures() {
+		return SMW_NAMESPACE_QUERY;
+	}
+
 }
 
 /**
@@ -425,6 +484,7 @@ class SMWNamespaceDescription extends SMWDescription {
  * from the concrete domain.
  * In RDF, concrete domain predicates that define ranges (like "greater or 
  * equal to") are not directly available.
+ * @ingroup SMWQuery
  */
 class SMWValueDescription extends SMWDescription {
 	protected $m_datavalue;
@@ -484,6 +544,7 @@ class SMWValueDescription extends SMWDescription {
  * values for some n-ary property. NULL values are to be used for 
  * unspecifed values. Corresponds to the built-in support for n-ary 
  * properties, i.e. can be viewed as a macro in OWL and RDF.
+ * @ingroup SMWQuery
  */
 class SMWValueList extends SMWDescription {
 	protected $m_descriptions;
@@ -596,6 +657,7 @@ class SMWValueList extends SMWDescription {
  * must be satisfied (AND, conjunction).
  *
  * Corresponds to conjunction in OWL and SPARQL. Not available in RDFS.
+ * @ingroup SMWQuery
  */
 class SMWConjunction extends SMWDescription {
 	protected $m_descriptions;
@@ -663,6 +725,14 @@ class SMWConjunction extends SMWDescription {
 		return $depth;
 	}
 
+	public function getQueryFeatures() {
+		$result = SMW_CONJUNCTION_QUERY;
+		foreach ($this->m_descriptions as $desc) {
+			$result = $result | $desc->getQueryFeatures();
+		}
+		return $result;
+	}
+
 	public function prune(&$maxsize, &$maxdepth, &$log) {
 		if ($maxsize <= 0) {
 			$log[] = $this->getQueryString();
@@ -698,6 +768,7 @@ class SMWConjunction extends SMWDescription {
  * must be satisfied (OR, disjunction).
  *
  * Corresponds to disjunction in OWL and SPARQL. Not available in RDFS.
+ * @ingroup SMWQuery
  */
 class SMWDisjunction extends SMWDescription {
 	protected $m_descriptions;
@@ -795,6 +866,14 @@ class SMWDisjunction extends SMWDescription {
 		return $depth;
 	}
 
+	public function getQueryFeatures() {
+		$result = SMW_DISJUNCTION_QUERY;
+		foreach ($this->m_descriptions as $desc) {
+			$result = $result | $desc->getQueryFeatures();
+		}
+		return $result;
+	}
+
 	public function prune(&$maxsize, &$maxdepth, &$log) {
 		if ($maxsize <= 0) {
 			$log[] = $this->getQueryString();
@@ -832,12 +911,13 @@ class SMWDisjunction extends SMWDescription {
  * Corresponds to existential quatification ("some" restriction) on concrete properties
  * in OWL. In conjunctive queries (OWL) and SPARQL (RDF), it is represented by using 
  * variables in the object part of such properties.
+ * @ingroup SMWQuery
  */
 class SMWSomeProperty extends SMWDescription {
 	protected $m_description;
 	protected $m_property;
 
-	public function SMWSomeProperty(Title $property, SMWDescription $description) {
+	public function SMWSomeProperty(SMWPropertyValue $property, SMWDescription $description) {
 		$this->m_property = $property;
 		$this->m_description = $description;
 	}
@@ -854,9 +934,9 @@ class SMWSomeProperty extends SMWDescription {
 		$subdesc = $this->m_description->getQueryString(true);
 		$sep = ($this->m_description instanceof SMWSomeProperty)?'.':'::'; // use property chain syntax
 		if ($asvalue) {
-			return $this->m_property->getText() . $sep . $subdesc;
+			return $this->m_property->getWikiValue() . $sep . $subdesc;
 		} else {
-			return '[[' . $this->m_property->getText() . $sep . $subdesc . ']]';
+			return '[[' . $this->m_property->getWikiValue() . $sep . $subdesc . ']]';
 		}
 	}
 
@@ -872,6 +952,10 @@ class SMWSomeProperty extends SMWDescription {
 		return 1+$this->getDescription()->getDepth();
 	}
 
+	public function getQueryFeatures() {
+		return SMW_PROPERTY_QUERY | $this->m_description->getQueryFeatures();
+	}
+
 	public function prune(&$maxsize, &$maxdepth, &$log) {
 		if (($maxsize <= 0)||($maxdepth <= 0)) {
 			$log[] = $this->getQueryString();
