@@ -3,17 +3,25 @@
  * @author Markus Krötzsch
  *
  * This page shows all wanted properties (used but not having a page).
+ * @file
+ * @ingroup SMWSpecialPage
+ * @ingroup SpecialPage
  */
 
 function smwfDoSpecialWantedProperties() {
+	global $wgOut;
 	wfProfileIn('smwfDoSpecialWantedProperties (SMW)');
 	list( $limit, $offset ) = wfCheckLimits();
 	$rep = new SMWWantedPropertiesPage();
 	$result = $rep->doQuery( $offset, $limit );
+	SMWOutputs::commitToOutputPage($wgOut); // make sure locally collected output data is pushed to the output!
 	wfProfileOut('smwfDoSpecialWantedProperties (SMW)');
 	return $result;
 }
 
+/**
+ * @ingroup SMWQuery
+ */
 class SMWWantedPropertiesPage extends SMWQueryPage {
 
 	function getName() {
@@ -30,12 +38,18 @@ class SMWWantedPropertiesPage extends SMWQueryPage {
 	}
 
 	function getPageHeader() {
+		wfLoadExtensionMessages('SemanticMediaWiki');
 		return '<p>' . wfMsg('smw_wantedproperties_docu') . "</p><br />\n";
 	}
 
 	function formatResult( $skin, $result ) {
 		global $wgLang, $wgExtraNamespaces;
-		$proplink = $skin->makeLinkObj($result[0], $result[0]->getText(), 'action=view');
+		if ($result[0]->isUserDefined()) {
+			$proplink = $skin->makeLinkObj($result[0]->getWikiPageValue()->getTitle(), $result[0]->getWikiValue(), 'action=view');
+		} else {
+			$proplink = $result[0]->getLongHTMLText($skin);
+		}
+		wfLoadExtensionMessages('SemanticMediaWiki');
 		return wfMsg('smw_wantedproperty_template', $proplink, $result[1]);
 	}
 	
