@@ -4,6 +4,9 @@
  *
  * This special page of the Semantic MediaWiki extension displays some 
  * statistics about properties.
+ * @file
+ * @ingroup SMWSpecialPage
+ * @ingroup SpecialPage
  */
 
 function smwfExecuteSemanticStatistics() {
@@ -32,6 +35,7 @@ function smwfExecuteSemanticStatistics() {
 	$wpurl = $sp->getFullURL();
 	$sp = Title::makeTitle( NS_SPECIAL, 'UnusedProperties');
 	$upurl = $sp->getFullURL();
+	wfLoadExtensionMessages('SemanticMediaWiki');
 	$out = wfMsg('smw_semstats_text',
 	             $wgLang->formatNum($semstats['PROPUSES']), $wgLang->formatNum($semstats['USEDPROPS']),
 	             $purl, $wgLang->formatNum($property_pages), $wgLang->formatNum($semstats['DECLPROPS']),
